form.incubator-inline-form {
    select {
        width: auto;
        min-width: unset;
        max-width: unset;
        border: none;
        padding: 0 0.5em;
        margin: 0;
        line-height: unset;
        box-sizing:border-box;
        height: 1.5em;

        &:hover {
            border: none;
        }

        &:focus {
            border: none;
        }
    }
    input[type=submit] {
        padding: 0 0.25em;
        margin: 0 1em 0 0;
        border: none;
        line-height: unset;
        box-sizing: border-box;
    }
}
