<?php

// autoload_static.php @generated by Composer

namespace Composer\Autoload;

class ComposerStaticInit3840543b1f6e6c0c88a270b469cc4baa
{
    public static $prefixLengthsPsr4 = array (
        'g' => 
        array (
            'gipfl\\ZfDb\\' => 11,
            'gipfl\\ZfDbStore\\' => 16,
            'gipfl\\Web\\' => 10,
            'gipfl\\Translation\\' => 18,
            'gipfl\\SystemD\\' => 14,
            'gipfl\\Protocol\\NetString\\' => 25,
            'gipfl\\Protocol\\JsonRpc\\' => 23,
            'gipfl\\Protocol\\Generic\\' => 23,
            'gipfl\\Protocol\\Exception\\' => 25,
            'gipfl\\Log\\' => 10,
            'gipfl\\LinuxHealth\\' => 18,
            'gipfl\\IcingaWeb2\\' => 17,
            'gipfl\\IcingaCliDaemon\\' => 22,
            'gipfl\\Diff\\' => 11,
            'gipfl\\Cli\\' => 10,
            'gipfl\\Calendar\\' => 15,
        ),
        'P' => 
        array (
            'Psr\\Log\\' => 8,
        ),
    );

    public static $prefixDirsPsr4 = array (
        'gipfl\\ZfDb\\' => 
        array (
            0 => __DIR__ . '/..' . '/gipfl/zfdb/src',
        ),
        'gipfl\\ZfDbStore\\' => 
        array (
            0 => __DIR__ . '/..' . '/gipfl/zfdbstore/src',
        ),
        'gipfl\\Web\\' => 
        array (
            0 => __DIR__ . '/..' . '/gipfl/web/src',
        ),
        'gipfl\\Translation\\' => 
        array (
            0 => __DIR__ . '/..' . '/gipfl/translation/src',
        ),
        'gipfl\\SystemD\\' => 
        array (
            0 => __DIR__ . '/..' . '/gipfl/systemd/src',
        ),
        'gipfl\\Protocol\\NetString\\' => 
        array (
            0 => __DIR__ . '/..' . '/gipfl/protocol-netstring/src',
        ),
        'gipfl\\Protocol\\JsonRpc\\' => 
        array (
            0 => __DIR__ . '/..' . '/gipfl/protocol-jsonrpc/src',
        ),
        'gipfl\\Protocol\\Generic\\' => 
        array (
            0 => __DIR__ . '/..' . '/gipfl/protocol/src/Generic',
        ),
        'gipfl\\Protocol\\Exception\\' => 
        array (
            0 => __DIR__ . '/..' . '/gipfl/protocol/src/Exception',
        ),
        'gipfl\\Log\\' => 
        array (
            0 => __DIR__ . '/..' . '/gipfl/log/src',
        ),
        'gipfl\\LinuxHealth\\' => 
        array (
            0 => __DIR__ . '/..' . '/gipfl/linux-health/src',
        ),
        'gipfl\\IcingaWeb2\\' => 
        array (
            0 => __DIR__ . '/..' . '/gipfl/icingaweb2/src',
        ),
        'gipfl\\IcingaCliDaemon\\' => 
        array (
            0 => __DIR__ . '/..' . '/gipfl/icinga-cli-daemon/src',
        ),
        'gipfl\\Diff\\' => 
        array (
            0 => __DIR__ . '/..' . '/gipfl/diff/src',
        ),
        'gipfl\\Cli\\' => 
        array (
            0 => __DIR__ . '/..' . '/gipfl/cli/src',
        ),
        'gipfl\\Calendar\\' => 
        array (
            0 => __DIR__ . '/..' . '/gipfl/calendar/src',
        ),
        'Psr\\Log\\' => 
        array (
            0 => __DIR__ . '/..' . '/psr/log/Psr/Log',
        ),
    );

    public static function getInitializer(ClassLoader $loader)
    {
        return \Closure::bind(function () use ($loader) {
            $loader->prefixLengthsPsr4 = ComposerStaticInit3840543b1f6e6c0c88a270b469cc4baa::$prefixLengthsPsr4;
            $loader->prefixDirsPsr4 = ComposerStaticInit3840543b1f6e6c0c88a270b469cc4baa::$prefixDirsPsr4;

        }, null, ClassLoader::class);
    }
}
